@@ -268,8 +268,24 @@ func (p *parser) parseMappingValue(ctx *context) (ast.Node, error) {
 		antk = ctx.afterNextNotCommentToken()
 	}
 	if len(node.Values) == 1 {
+		mapKeyCol := mvnode.Key.GetToken().Position.Column
+		commentTk := ctx.nextToken()
+		if commentTk != nil && commentTk.Type == token.CommentType && mapKeyCol <= commentTk.Position.Column {
+			// If the comment is in the same or deeper column as the last element column in map value,
+			// treat it as a footer comment for the last element.
+			comment := p.parseFootComment(ctx, mapKeyCol)
+			mvnode.FootComment = comment
+		}
 		return mvnode, nil
 	}
+	mapCol := node.GetToken().Position.Column
+	commentTk := ctx.nextToken()
+	if commentTk != nil && commentTk.Type == token.CommentType && mapCol <= commentTk.Position.Column {
+		// If the comment is in the same or deeper column as the last element column in map value,
+		// treat it as a footer comment for the last element.
+		comment := p.parseFootComment(ctx, mapCol)
+		node.FootComment = comment
+	}
 	return node, nil
 }
 
@@ -299,9 +315,9 @@ func (p *parser) parseSequenceEntry(ctx *context) (*ast.SequenceNode, error) {
 		}
 		if comment != nil {
 			comment.SetPath(ctx.withIndex(uint(len(sequenceNode.Values))).path)
-			sequenceNode.ValueComments = append(sequenceNode.ValueComments, comment)
+			sequenceNode.ValueHeadComments = append(sequenceNode.ValueHeadComments, comment)
 		} else {
-			sequenceNode.ValueComments = append(sequenceNode.ValueComments, nil)
+			sequenceNode.ValueHeadComments = append(sequenceNode.ValueHeadComments, nil)
 		}
 		sequenceNode.Values = append(sequenceNode.Values, value)
 		tk = ctx.nextNotCommentToken()
@@ -316,6 +332,13 @@ func (p *parser) parseSequenceEntry(ctx *context) (*ast.SequenceNode, error) {
 		}
 		ctx.progressIgnoreComment(1)
 	}
+	commentTk := ctx.nextToken()
+	if commentTk != nil && commentTk.Type == token.CommentType && curColumn <= commentTk.Position.Column {
+		// If the comment is in the same or deeper column as the last element column in sequence value,
+		// treat it as a footer comment for the last element.
+		comment := p.parseFootComment(ctx, curColumn)
+		sequenceNode.FootComment = comment
+	}
 	return sequenceNode, nil
 }
 
@@ -527,6 +550,26 @@ func (p *parser) parseCommentOnly(ctx *context) *ast.CommentGroupNode {
 	return ast.CommentGroup(commentTokens)
 }
 
+func (p *parser) parseFootComment(ctx *context, col int) *ast.CommentGroupNode {
+	commentTokens := []*token.Token{}
+	for {
+		ctx.progressIgnoreComment(1)
+		commentTokens = append(commentTokens, ctx.currentToken())
+
+		nextTk := ctx.nextToken()
+		if nextTk == nil {
+			break
+		}
+		if nextTk.Type != token.CommentType {
+			break
+		}
+		if col > nextTk.Position.Column {
+			break
+		}
+	}
+	return ast.CommentGroup(commentTokens)
+}
+
 func (p *parser) parseComment(ctx *context) (ast.Node, error) {
 	group := p.parseCommentOnly(ctx)
 	node, err := p.parseToken(ctx, ctx.currentToken())
